<?php

/*
 * @file hotel_discounts.admin.inc
 * Provides admin screens for hotel discounts
 * @copyright Copyright(c) 2010 Lee Rowlands
 * @license GPL v2 http://www.fsf.org/licensing/licenses/gpl.html
 * @author Lee Rowlands leerowlands at rowlands-bcs dot com
 *
 */

/**
 * Form builder for hotel_discounts list
 * We use a form to build a table
 * so other modules can modify it with hook_form_alter
 * but theme it as a table
 * @see theme_hotel_discounts_list_form
*/
function hotel_discounts_list_form($form_state) {
  $discounts = hotel_discounts_get_discounts();
  $form = array();
  foreach ($discounts as $discount) {
    $form['discounts'][$discount->hdid] = array(
      '#value' => $discount,
      '#type' => 'value'
    );
  }

  return $form;
}

/**
 * Form builder for hotel_discounts discount intervals
 * We use a form to build a table
 * so other modules can modify it with hook_form_alter
 * but theme it as a table
 * @see theme_hotel_discounts_intervals_list_form
*/
function hotel_discounts_intervals_list_form($form_state) {
  $intervals = hotel_discounts_get_discount_intervals();
  $form = array();
  foreach ($intervals as $interval) {
    $form['intervals'][$interval->hdiid] = array(
      '#value' => $interval,
      '#type' => 'value'
    );
  }

  return $form;
}


/**
 * Form builder for hotel_discounts discount intervals
 * We use a form to build a table
 * so other modules can modify it with hook_form_alter
 * but theme it as a table
 * @param $form_state array std FAPI form state var
 * @param $interval object interval being edited
*/
function hotel_discounts_intervals_form($form_state, $interval = FALSE) {
  if (!$interval) {
    drupal_set_title(t('Add discount interval'));
  }
  $form = array();

  $form['hdiid'] = array(
    '#type' => 'value',
    '#value' => $interval ? $interval->hdiid : NULL
  );

  $form['name'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Interval name'),
    '#default_value' => $interval ? $interval->name : FALSE,
    '#size'          => 60,
    '#description'   => t('Enter a name for your interval eg Week, Two Months'),
    '#maxlength'     => 50,
    '#required'      => TRUE,
  );

  $form['days'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Days in interval'),
    '#default_value' => $interval ? $interval->days : 0,
    '#size'          => 60,
    '#description'   => t('Enter number of days in interval eg enter 7 for a week, 14 for a fortnight'),
    '#maxlength'     => 50,
    '#required'      => TRUE,
  );

  $form['months'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Months in interval'),
    '#default_value' => $interval ? $interval->months : 0,
    '#size'          => 60,
    '#description'   => t('Enter number of months in interval eg enter 12 for 1 year, 3 for a quarter'),
    '#maxlength'     => 50,
    '#required'      => TRUE,
  );

  $form['submit'] = array('#type' => 'submit', '#value' => t('Save'));

  return $form;
}

/**
 * Validation handler for hotel_discounts_intervals_form
*/
function hotel_discounts_intervals_form_validate($form, &$form_state) {
  $ints = array('days', 'months');
  foreach ($ints as $int) {
    if (!is_numeric($form_state['values'][$int]) ||
        !is_int((int)$form_state['values'][$int])) {
      form_set_error($int, t('Please enter a valid integer for !field', array('!field' => $int)));
    }
  }
}

/**
 * Submit handler for hotel_discounts_intervals_form
*/
function hotel_discounts_intervals_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $update = array();
  if ($values['hdiid']) {
    $update[] = 'hdiid';
  }
  drupal_write_record('hotel_discounts_intervals', $values, $update);
  drupal_set_message(t('Saved discount interval: <em>@name</em>', array('@name' => $values['name'])));
  $form_state['redirect'] = 'admin/store/hotel_booking/discount-intervals';
}

/**
 * Form builder for delete discount
 * @param $form_state array std form state var
 * @param $discount object discount object @see hotel_discount_load
*/
function hotel_discounts_delete_form($form_state, $discount) {
  drupal_set_title(t('Delete @name', array('@name' => $discount->name)));

  $form['discount'] = array(
    '#type' => 'value',
    '#value' => $discount,
  );

  $form = confirm_form($form,
    t('Delete @name?', array('@name' => $discount->name)),
    'admin/store/hotel_booking/discounts/'. $discount->hdid,
    t('This action cannot be undone'),
    t('Delete'),
    t('Cancel')
  );

  return $form;
}

/**
 * Submit handler to delete discount
*/
function hotel_discounts_delete_form_submit($form_state, &$form_state) {
  if ($form_state['values']['confirm']) {
    db_query('DELETE FROM {hotel_discounts} WHERE hdid = %d', $form_state['values']['discount']->hdid);
    db_query('DELETE FROM {hotel_discounts_date} WHERE hdid = %d', $form_state['values']['discount']->hdid);
    db_query('DELETE FROM {hotel_discounts_node} WHERE hdid = %d', $form_state['values']['discount']->hdid);

    drupal_set_message(t('Discount "@name" has been deleted', array('@name' => $form_state['values']['discount']->name)));
  }

  $form_state['redirect'] = 'admin/store/hotel_booking/discounts';

}

/**
 * Form builder for delete discount intervals
 * @param $form_state array std form state var
 * @param $interval object discount interval object @see hotel_discount_interval_load
*/
function hotel_discounts_intervals_delete_form($form_state, $interval) {
  $discounts = db_result(db_query("SELECT COUNT(*)
                                  FROM {hotel_discounts}
                                  WHERE hdiid = %d",
                                  $interval->hdiid));
  if ($discounts && $discounts > 0) {
    drupal_set_message(t('You cannot delete this interval as it is associated with active discounts, please delete the discounts first.'), 'warning');
    drupal_goto('admin/store/hotel_booking/discounts');
  }

  drupal_set_title(t('Delete @name', array('@name' => $interval->name)));

  $form['interval'] = array(
    '#type' => 'value',
    '#value' => $interval,
  );

  $form = confirm_form($form,
    t('Delete @name?', array('@name' => $interval->name)),
    'admin/store/hotel_booking/discount-intervals/'. $interval->hdiid,
    t('This action cannot be undone'),
    t('Delete'),
    t('Cancel')
  );

  return $form;
}

/**
 * Submit handler to delete discount interval
*/
function hotel_discounts_intervals_delete_form_submit($form_state, &$form_state) {
  if ($form_state['values']['confirm']) {
    db_query('DELETE FROM {hotel_discounts_intervals} WHERE hdiid = %d', $form_state['values']['interval']->hdiid);

    drupal_set_message(t('Discount interval "@name" has been deleted',
                         array('@name' => $form_state['values']['interval']->name)));
  }

  $form_state['redirect'] = 'admin/store/hotel_booking/discount-intervals';

}

/**
 * Form builder to add/edit discount
 * @param $form_state array std fapi form state var
 * @param $discount object discount object @see hotel_discount_load
*/
function hotel_discounts_edit_form($form_state, $discount = FALSE) {
  $intervals = hotel_discounts_get_discount_intervals(TRUE);
  if (count($intervals) == 0) {
    drupal_set_message(t('You must setup a discount interval before you can add a discount.'));
    drupal_goto('admin/store/hotel_booking/discount-intervals/add');
  }
  if ($discount) {
    drupal_set_title(t('Edit @name', array('@name' => $discount->name)));
  }
  else {
    drupal_set_title(t('Add new discount'));
  }
  $form = array();
  if (!$form_state['storage']['removed']) {
    $form_state['storage']['removed'] = array();
  }
  else {
    drupal_set_message(t('Changes to removed items will not be made permanent until save is pressed.'));
  }

  $form['hdid'] = array(
    '#value' => $discount ? $discount->hdid: NULL,
    '#type' => 'value'
  );

  $form['name'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Discount name'),
    '#default_value' => $discount ? $discount->name : '',
    '#size'          => 60,
    '#description'   => t('Enter a name for this discount'),
    '#maxlength'     => 50,
    '#required'      => TRUE,
  );

  $form['type'] = array(
    '#type' => 'select',
    '#title' => t('Type of discount'),
    '#default_value' => $discount ? $discount->type: NULL,
    '#options' => hotel_discounts_type_names(),
    '#description' => t('Choose the way this discount will be applied. Note that the "per interval"
                        methods ignore any other rates and discount methods and <em>will always override a
                        shorter interval</em>. Eg if you setup a "per interval" price for a week as well as
                        a "per interval" price for a month. If the guest stays a month, they will be charged
                        the monthly price, the weekly price will be ignored. The "per interval" methods completely
                        ignore any base rates, rate modifiers or occupancy modifiers setup using the standard
                        uc_hotel module functionality. "Round up" means if they are staying 9 days and your discount
                        interval is a week (7 days), the guest will be charged for two weeks. "Round down" means they would
                        be charged for one week. "Apportion" means they would be charged 1 + 2/7 weeks.'),
  );

  $form['hdiid'] = array(
    '#type' => 'select',
    '#title' => t('Qualifying interval'),
    '#default_value' => $discount ? $discount->hdiid : NULL,
    '#options' => $intervals,
    '#description' => t('Choose the minimum stay required before the customer qualifies for this discount.'),
  );

  $form['rate'] = array(
    '#type'          => 'textfield',
    '#title'         => t('Rate'),
    '#default_value' => $discount ? $discount->rate : 0,
    '#size'          => 60,
    '#description'   => t('Enter the discount rate. The format entered will depend on the discount type. For a percent enter the percent as a fraction eg 60% = 0.6. For a flat charge enter the amount with no currency sign.'),
    '#maxlength'     => 10,
    '#required'      => TRUE,
  );

  $form['override'] = array(
    '#type' => 'checkbox',
    '#title' => t('Override shorter stay discounts'),
    '#default_value' => $discount ? $discount->override: FALSE,
    '#description' => t('Should this discount override shorter stay discounts. Eg if another discount applies for stays of 7 days or more
                        while this discount applies for 30 days, should this discount be used instead of the 7 days discount
                        if the stay length is 30 days. Uncheck this option if you want mutliple valid discounts to compound.')
  );

  $items = $discount->dates;

// the contents of $items will either come from the db or from $form_state
  if (isset($form_state['items'])) {
    $items = $form_state['items'] + (array)$items;
  }

  $form['items_wrapper'] = array(
    '#tree' => FALSE,
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#title' => t('Valid dates for this discount'),
    '#description' => t('Enter the valid dates for this discount'),
    '#prefix' => '<div class="clear-block" id="hotel-discounts-items-wrapper">',
    '#suffix' => '</div>',
  );

  $form['items_wrapper']['items'] = array(
    '#tree' => TRUE,
    '#prefix' => '<div id="hotel-discounts-items">',
    '#suffix' => '</div>',
    '#theme' => 'hotel_discounts_date_items',
  );

  if (count($items) == 0 ||
      count($items) == count($form_state['storage']['removed'])) {
    //we have no items or all are to be removed
    $items = array(
      0 => array(),
    );
  }

  // Add the current items to the form.
  foreach ($items as $delta => $item) {
    if (in_array($item['hddid'], $form_state['storage']['removed'])) {
      continue; //don't do the removed ones
    }
    $item['delta'] = $delta;
    $item['hdid'] = $discount->hdid;
    $form['items_wrapper']['items'][$delta] = hotel_discounts_date_row($item);
  }

  $form['items_wrapper']['add_another'] = array(
    '#type' => 'submit',
    '#value' => t('Add another'),
    '#attributes' => array('class' => 'hotel-discounts-new-item',
                           'title' => t('Click here to add new date range.')),
    '#submit' => array('hotel_discounts_add_date_submit'), // If no javascript action.
    '#ahah' => array(
      'path' => 'hotel-discounts/ahah/items_wrapper/items',
      'wrapper' => 'hotel-discounts-items',
      'method' => 'replace',
      'effect' => 'fade',
    ),
  );

  //the save button
  $form['submit'] = array('#type' => 'submit', '#value' => t('Save'));

  $form['#cache'] = TRUE;

  return $form;
}

/**
 * Form builder to generate a row of values for the date ranges
*/
function hotel_discounts_date_row($item) {
  $form['#tree'] = TRUE;
  $delta = $item['delta'];

  $form['hddid'] = array(
    '#type' => 'value',
    '#value' => $item['hddid']
  );

  $form['hdid'] = array(
    '#type' => 'value',
    '#value' => $item['hdid']
  );

  $form['start_dt'] = array(
    '#type' => 'date_popup',
    '#title' => t('Start Date'),
    '#required' => TRUE,
    '#size' => 14,
    '#default_value' => $item['start_dt'],
    '#date_type' => DATE_ISO,
    '#date_timezone' => date_default_timezone_name(),
    '#date_format' => variable_get('uc_date_format_default', 'd M Y'),
    '#date_year_range' => '-0:+1',
    '#description' => t('Start date of discount validity')
  );

  $form['end_dt'] = array(
    '#type' => 'date_popup',
    '#title' => t('End Date'),
    '#required' => FALSE,
    '#size' => 14,
    '#default_value' => $item['end_dt'],
    '#date_type' => DATE_ISO,
    '#date_timezone' => date_default_timezone_name(),
    '#date_format' => variable_get('uc_date_format_default', 'd M Y'),
    '#date_year_range' => '-0:+1',
    '#description' => t('End date of discount validity. Leave blank for none')
  );

  $form['remove'] = array(
    '#type' => 'submit',
    '#prefix' => '<div>',
    '#suffix' => '<label for="edit-remove">'. t('Remove') .'</label></div>',
    '#value' => 'remove_'. $delta,
    '#attributes' => array('class' => 'delete-item', 'title' => t('Click here to delete this item.')),
    '#submit' => array('hotel_discounts_remove_date_submit'),
    '#ahah' => array(
      'path' => 'hotel-discounts/ahah/items_wrapper/items',
      'wrapper' => 'hotel-discounts-items',
      'method' => 'replace',
      'effect' => 'fade',
    ),
  );

  return $form;
}

/**
 * Ahah handler for adding dates
*/
function hotel_discounts_ahah() {
  //get args
  $elements = func_get_args();
  //we set submitted as TRUE so prevent _form_builder_ie_cleanup() from triggering
  //submit on ahah that does not involve a button
  $form_state = array('storage' => NULL, 'submitted' => FALSE);
  $form_build_id = $_POST['form_build_id'];
  $form = form_get_cache($form_build_id, $form_state);
  $args = $form['#parameters'];
  $form_id = array_shift($args);
  $form['#post'] = $_POST;
  $form['#redirect'] = FALSE;
  $form['#programmed'] = FALSE;
  $form_state['post'] = $_POST;
  drupal_process_form($form_id, $form, $form_state);
  $form = drupal_rebuild_form($form_id, $form_state, $args, $form_build_id);
  foreach ($elements as $element) {
    $form = $form[$element];
  }
  if (!$form) {
    watchdog('hotel_discounts', 'Ahah method was called with elements not found in resultant form', array(), WATCHDOG_ERROR);
    $form = array(
      'error' => array(
        '#value' => t('An error occured while processing your request, please try again. If the problem persists, please contact the site administrator')
      )
    );
  }
  unset($form['#prefix'], $form['#suffix']); // Prevent duplicate wrappers.
  $javascript = drupal_add_js(NULL, NULL, 'header');
  // Loop through the JS settings and find the settings needed for our buttons.
  $new_ahah_settings = $new_date_settings = array();
  if (isset($javascript['setting'])) {
    foreach ($javascript['setting'] as $settings) {
      if (isset($settings['ahah'])) {
        foreach ($settings['ahah'] as $id => $ahah_settings) {
          $new_ahah_settings[$id] = $ahah_settings;
        }
      }
      if (isset($settings['datePopup'])) {
        foreach ($settings['datePopup'] as $id => $date_settings) {
          $new_date_settings[$id] = $date_settings;
        }
      }
    }
  }

  // Add the AHAH settings needed for our new elements.
  if (!empty($new_ahah_settings)) {
    $output = '<script type="text/javascript">jQuery.extend(Drupal.settings.ahah, '. drupal_to_js($new_ahah_settings) .');</script>';
  }

  // Add the date popup settings needed for our new elements.
  if (!empty($new_date_settings)) {
    $output = '<script type="text/javascript">jQuery.extend(Drupal.settings.datePopup, '. drupal_to_js($new_date_settings) .');</script>';
  }

  drupal_json(array(
    'status'   => TRUE,
    'data'     => theme('status_messages') . $output . drupal_render($form),
    'settings' => call_user_func_array('array_merge_recursive', $javascript['setting'])
  ));
}

/**
 * Submit handler to add a date range
*/
function hotel_discounts_add_date_submit($form, &$form_state) {
  $items = hotel_discounts_form_state_to_concessions($form_state);
  // Add an empty tab at the end.
  $items[] = array();

  unset($form_state['submit_handlers']);
  form_execute_handlers('submit', $form, $form_state);
  $form_state['items'] = $items;
  $form_state['rebuild'] = TRUE;
}

/**
 * Submit handler to remove a date range
*/
function hotel_discounts_remove_date_submit($form, &$form_state) {
  // Get the pane delta for the clicked button.
  $delta = $form_state['clicked_button']['#parents'][1];
  // Copy the form_state because unset() does not work on variable
  // that is passed by reference.
  $form_state_copy = $form_state;
  $remove = $form_state_copy['values']['items'][$delta];
  unset($form_state_copy['values']['items'][$delta]);
  $items = hotel_discounts_form_state_to_concessions($form_state_copy);

  unset($form_state['submit_handlers']);
  form_execute_handlers('submit', $form, $form_state);
  $form_state['items'] = $items;
  if (is_array($form_state['storage']['removed'])) {
    $form_state['storage']['removed'][] = $remove['hddid'];
  }
  else {
    $form_state['storage']['removed'] = array($remove['hddid']);
  }
  $form_state['rebuild'] = TRUE;

}

/**
 * Utility to convert form state vars to items
 * not much happening here in this case
*/
function hotel_discounts_form_state_to_concessions($form_state) {
  return $form_state['values']['items'];
}

/**
 * Default implementation of theme_hotel_discounts_list_form
 * @ingroup themeable
*/
function theme_hotel_discounts_list_form($form) {
  $header = array(
    t('Name'),
    t('Threshold/Interval'),
    t('Type'),
    t('Valid dates'),
    t('Actions'),
  );

  $rows = array();
  foreach (element_children($form['discounts']) as $key) {
    $discount = $form['discounts'][$key]['#value'];
    $rows[] = array(
      check_plain($discount->name),
      $discount->threshold_display,
      $discount->type_display,
      $discount->date_display,
      l(t('Edit'), 'admin/store/hotel_booking/discounts/'. $discount->hdid) .' | '.
      l(t('Delete'), 'admin/store/hotel_booking/discounts/'. $discount->hdid .'/delete'),
    );
  }
  if (count($rows) == 0) {
    $rows[] = array(array('data' => t('No length of stay discounts exist, please add one.'), 'colspan' => 5));
  }

  $output = theme('table', $header, $rows);

  $output .= l(t('Add length of stay discount'), 'admin/store/hotel_booking/discounts/add');
  return $output;
}

/**
 * Default implementation of theme_hotel_discounts_list_form
 * @ingroup themeable
*/
function theme_hotel_discounts_intervals_list_form($form) {
  $header = array(
    t('Name'),
    t('Days'),
    t('Months'),
    t('Actions'),
  );

  $rows = array();
  foreach (element_children($form['intervals']) as $key) {
    $interval = $form['intervals'][$key]['#value'];
    $rows[] = array(
      check_plain($interval->name),
      $interval->days,
      $interval->months,
      l(t('Edit'), 'admin/store/hotel_booking/discount-intervals/'. $interval->hdiid) .' | '.
      l(t('Delete'), 'admin/store/hotel_booking/discount-intervals/'. $interval->hdiid .'/delete'),
    );
  }
  if (count($rows) == 0) {
    $rows[] = array(array('data' => t('No discount intervals exist, please add one.'), 'colspan' => 4));
  }

  $output = theme('table', $header, $rows);

  $output .= l(t('Add discount interval'), 'admin/store/hotel_booking/discount-intervals/add');
  return $output;
}

/**
 * Default implementation of theme_hotel_discounts_date_items
 * @ingroup themeable
*/
function theme_hotel_discounts_date_items($form) {
  $rows = array();
  $headers = array(
    t('Start Date'),
    t('End Date')
  );

  foreach (element_children($form) as $key) {
    // Build the table row.
    $row = array(
      array('data' => drupal_render($form[$key]['start_dt']), 'class' => 'hotel-discounts-date-start-dt'),
      array('data' => drupal_render($form[$key]['end_dt']), 'class' => 'hotel-discounts-date-end-dt'),
      array('data' => drupal_render($form[$key]['remove']), 'class' => 'hotel-discounts-date-remove'),
    );

    $rows[] = $row;
  }

  $output .= theme('table', $headers, $rows, array('id' => 'hotel-discounts-dates-table'));
  $output .= drupal_render($form);

  drupal_add_css(drupal_get_path('module', 'hotel_discounts') .'/hotel_discounts.css');

  return $output;
}

/**
 * Validation handler for hotel_discounts_edit_form
*/
function hotel_discounts_edit_form_validate($form, &$form_state) {
  if (!is_numeric($form_state['values']['rate'])) {
    form_set_error('rate', t('Please enter a valid number for the rate'));
  }
  if ($form_state['values']['type'] <= 1 && //percent rate
      $form_state['values']['rate'] > 1) {
    form_set_error('rate', t('Please provide the percent rate as a fraction eg 60% = 0.6'));
  }
  if (!count($form_state['values']['items'])) {
    form_set_error('', t('You must provide at least one valid date range'));
  }
}

/**
 * Submit handler for hotel_discounts_edit_form
*/
function hotel_discounts_edit_form_submit($form, &$form_state) {
  if ($form_state['values']['op'] == t('Save')) {
    $update = array();
    if ($form_state['values']['hdid']) {
      $update = array('hdid');
    }
    $items = $form_state['values']['items'];
    unset($form_state['values']['items']);//strip this so we have a 1D record
    drupal_write_record('hotel_discounts', $form_state['values'], $update);
    db_query("DELETE FROM {hotel_discounts_date}
             WHERE hdid = %d", $form_state['values']['hdid']);
    foreach ($items as $item) {
      $item['hdid'] = $form_state['values']['hdid']; //this was either submitted by form or set by drupal_write_record
      drupal_write_record('hotel_discounts_date', $item);
    }
    drupal_set_message(t('Saved your discount: <em>@name</em>', array('@name' => $form_state['values']['name'])));
    drupal_goto('admin/store/hotel_booking/discounts');
  }
}
